#* This file is part of the MOOSE framework
#* https://www.mooseframework.org
#*
#* All rights reserved, see COPYRIGHT for full restrictions
#* https://github.com/idaholab/moose/blob/master/COPYRIGHT
#*
#* Licensed under LGPL 2.1, please see LICENSE for details
#* https://www.gnu.org/licenses/lgpl-2.1.html

#  This means the job was killed with a signal. The signal is given by X modulo 256. For example an exit value of
#  137 means the job's top process was killed with signal 9 (137 % 128 = 9).
KILL_SIGNALS = { '1' : 'SIGHUP:terminate process - terminal line hangup',
                 '2' : 'SIGINT:terminate process - interrupt program',
                 '3' : 'SIGQUIT:create core image - quit program',
                 '4' : 'SIGILL:create core image - illegal instruction',
                 '5' : 'SIGTRAP:create core image - trace trap',
                 '6' : 'SIGABRT:create core image - abort program (formerly SIGIOT)',
                 '7' : 'SIGEMT:create core image - emulate instruction executed',
                 '8' :  'SIGFPE:create core image - floating-point exception',
                 '9' :  'SIGKILL:terminate process - kill program',
                 '10' :  'SIGBUS:create core image - bus error',
                 '11' :  'SIGSEGV:create core image - segmentation violation',
                 '12' :  'SIGSYS:create core image - non-existent system call invoked',
                 '13' :  'SIGPIPE:terminate process - write on a pipe with no reader',
                 '14' :  'SIGALRM:terminate process - real-time timer expired',
                 '15' :  'SIGTERM:terminate process - software termination signal',
                 '16' :  'SIGURG:discard signal - urgent condition present on socket',
                 '17' :  'SIGSTOP:stop process - stop (cannot be caught or ignored)',
                 '18' :  'SIGTSTP:stop process - stop signal generated from keyboard',
                 '19' :  'SIGCONT:discard signal - continue after stop',
                 '20' :  'SIGCHLD:discard signal - child status has changed',
                 '21' :  'SIGTTIN:stop process - background read attempted from control terminal',
                 '22' :  'SIGTTOU:stop process - background write attempted to control terminal',
                 '23' :  'SIGIO:discard signal - I/O is possible on a descriptor (see fcntl(2))',
                 '24' :  'SIGXCPU:terminate process - cpu time limit exceeded (see setrlimit(2))',
                 '25' :  'SIGXFSZ:terminate process - file size limit exceeded (see setrlimit(2))',
                 '26' :  'SIGVTALRM:terminate process - virtual time alarm (see setitimer(2))',
                 '27' :  'SIGPROF:terminate process - profiling timer alarm (see setitimer(2))',
                 '28' :  'SIGWINCH:discard signal - Window size change',
                 '29' :  'SIGINFO:discard signal - status request from keyboard',
                 '30' :  'SIGUSR1:terminate process - User defined signal 1',
                 '31' :  'SIGUSR2:terminate process - User defined signal 2' }

# These errors come from src/include/job.h for OpenPBS
# Negative exit status indicates that the job could not be executed.
PBS_User_EXITCODES = { '0' : 'JOB_EXEC_OK:Job execution was successful',
                      '-1' : 'JOB_EXEC_FAIL1:Job execution failed, before files, no retry',
                      '-2' : 'JOB_EXEC_FAIL2:Job execution failed, after files, no retry',
                      '-3' : 'JOB_EXEC_RETRY:Job execution failed, do retry',
                      '-4' : 'JOB_EXEC_INITABT:Job aborted on MoM initialization',
                      '-5' : 'JOB_EXEC_INITRST:Job aborted on MoM initialization, checkpoint, no migrate',
                      '-6' : 'JOB_EXEC_INITRMG:Job aborted on MoM initialization, checkpoint, ok migrate',
                      '-7' : 'JOB_EXEC_BADRESRT:Job restart failed',
                      '-10' : 'JOB_EXEC_FAILUID:Invalid UID/GID for job',
                      '-11' : 'JOB_EXEC_RERUN:Job was rerun',
                      '-12' : 'JOB_EXEC_CHKP:Job was checkpointed and killed',
                      '-13' : 'JOB_EXEC_FAIL_PASSWORD:Job failed due to a bad password',
                      '-14' : 'JOB_EXEC_RERUN_ON_SIS_FAIL:Job was requeued (if rerunnable) or deleted (if not) due to a communication failure between the primary execution host MoM and a Sister',
                      '-15' : 'JOB_EXEC_QUERST:Requeue job for restart from checkpoint',
                      '-16' : 'JOB_EXEC_FAILHOOK_RERUN:Job execution failed due to hook rejection; requeue for later retry',
                      '-17' : 'JOB_EXEC_FAILHOOK_DELETE:Job execution failed due to hook rejection; delete the job at end',
                      '-18' : 'JOB_EXEC_HOOK_RERUN:A hook requested for job to be requeued',
                      '-19' : 'JOB_EXEC_HOOK_DELETE:A hook requested for job to be deleted',
                      '-20' : 'JOB_EXEC_RERUN_MS_FAIL:Job requeued because server could not contact the primary execution host MoM',
                      '-21' : 'JOB_EXEC_FAIL_SECURITY:Security breach in PBS directory',
                      '-22' : 'JOB_EXEC_HOOKERROR:Job exec failed due to unexpected exception or hook execution timed out',
                      '-23' : 'JOB_EXEC_FAIL_KRB5:Error no kerberos credentials supplied',
                      '-24' : 'JOB_EXEC_KILL_NCPUS_BURST:Job exec failed due to exceeding ncpus (burst)',
                      '-25' : 'JOB_EXEC_KILL_NCPUS_SUM:Job exec failed due to exceeding ncpus (sum)',
                      '-26' : 'JOB_EXEC_KILL_VMEM:Job exec failed due to exceeding vmem',
                      '-27' : 'JOB_EXEC_KILL_MEM:Job exec failed due to exceeding mem',
                      '-28' : 'JOB_EXEC_KILL_CPUT:Job exec failed due to exceeding cput',
                      '-29' : 'JOB_EXEC_KILL_WALLTIME:Job exec failed due to exceeding walltime',
                      '-30' : 'JOB_EXEC_JOINJOB:Job exec failed due to join job error' }

PBS_STATUSES = { '0' : 'UNKNOWN',
                 'B' : 'BEGUN',
                 'E' : 'EXITING',
                 'F' : 'FINISHED',
                 'H' : 'HELD',
                 'M' : 'MOVED',
                 'Q' : 'QUEUED',
                 'R' : 'RUNNING',
                 'S' : 'SUSPENDED',
                 'T' : 'TRANSIT',
                 'U' : 'USUSPENDED',
                 'W' : 'WAITING',
                 'X' : 'EXPIRED' }

# These errors come from src/include/pbs_errors.h
# This is the exit value of the top process in the job, typically the shell. This may be the exit value of the last command
# executed in the shell or the .logout script if the user has such a script (csh).
PBS_Server_EXITCODES = {   '1' : ' PBSE_UNKJOBID : Unknown Job Identifier',
                           '2' : ' PBSE_NOATTR : Undefined Attribute',
                           '3' : ' PBSE_ATTRRO : attempt to set READ ONLY attribute',
                           '4' : ' PBSE_IVALREQ : Invalid request',
                           '5' : ' PBSE_UNKREQ : Unknown batch request',
                           '6' : ' PBSE_TOOMANY : Too many submit retries',
                           '7' : ' PBSE_PERM : No permission',
                           '8' : ' PBSE_BADHOST : access from host not allowed',
                           '9' : ' PBSE_JOBEXIST : job already exists',
                           '10' : ' PBSE_SYSTEM : system error occurred',
                           '11' : ' PBSE_INTERNAL : internal server error occurred',
                           '12' : ' PBSE_REGROUTE : parent job of dependent in rte que',
                           '13' : ' PBSE_UNKSIG : unknown signal name',
                           '14' : ' PBSE_BADATVAL : bad attribute value',
                           '15' : ' PBSE_MODATRRUN : Cannot modify attrib in run state',
                           '16' : ' PBSE_BADSTATE : request invalid for job state',
                           '18' : ' PBSE_UNKQUE : Unknown queue name',
                           '19' : ' PBSE_BADCRED : Invalid Credential in request',
                           '20' : ' PBSE_EXPIRED : Expired Credential in request',
                           '21' : ' PBSE_QUNOENB : Queue not enabled',
                           '22' : ' PBSE_QACESS : No access permission for queue',
                           '23' : ' PBSE_BADUSER : Bad user  no password entry',
                           '24' : ' PBSE_HOPCOUNT : Max hop count exceeded',
                           '25' : ' PBSE_QUEEXIST : Queue already exists',
                           '26' : ' PBSE_ATTRTYPE : incompatable queue attribute type',
                           '27' : ' PBSE_OBJBUSY : Object Busy (not empty)',
                           '28' : ' PBSE_QUENBIG : Queue name too long',
                           '29' : ' PBSE_NOSUP : Feature/function not supported',
                           '30' : ' PBSE_QUENOEN : Cannot enable queue,needs add def',
                           '31' : ' PBSE_PROTOCOL : Batch Protocol error',
                           '32' : ' PBSE_BADATLST Bad attribute list structure',
                           '33' : ' PBSE_NOCONNECTS : No free connections',
                           '34' : ' PBSE_NOSERVER : No server to connect to',
                           '35' : ' PBSE_UNKRESC : Unknown resource',
                           '36' : ' PBSE_EXCQRESC : Job exceeds Queue resource limits',
                           '37' : ' PBSE_QUENODFLT : No Default Queue Defined',
                           '38' : ' PBSE_NORERUN : Job Not Rerunnable',
                           '39' : ' PBSE_ROUTEREJ : Route rejected by all destinations',
                           '40' : ' PBSE_ROUTEEXPD : Time in Route Queue Expired',
                           '41' : ' PBSE_MOMREJECT : Request to MOM failed',
                           '42' : ' PBSE_BADSCRIPT : (qsub) cannot access script file',
                           '43' : ' PBSE_STAGEIN : Stage In of files failed',
                           '44' : ' PBSE_RESCUNAV : Resources temporarily unavailable',
                           '45' : ' PBSE_BADGRP : Bad Group specified',
                           '46' : ' PBSE_MAXQUED : Max number of jobs in queue',
                           '47' : ' PBSE_CKPBSY : Checkpoint Busy, may be retries',
                           '48' : ' PBSE_EXLIMIT : Limit exceeds allowable',
                           '49' : ' PBSE_BADACCT : Bad Account attribute value',
                           '50' : ' PBSE_ALRDYEXIT : Job already in exit state',
                           '51' : ' PBSE_NOCOPYFILE : Job files not copied',
                           '52' : ' PBSE_CLEANEDOUT : unknown job id after clean init',
                           '53' : ' PBSE_NOSYNCMSTR : No Master in Sync Set',
                           '54' : ' PBSE_BADDEPEND : Invalid dependency',
                           '55' : ' PBSE_DUPLIST : Duplicate entry in List',
                           '56' : ' PBSE_DISPROTO : Bad DIS based Request Protocol',
                           '57' : ' PBSE_EXECTHERE : cannot execute there',
                           '58' : ' PBSE_SISREJECT : sister rejected',
                           '59' : ' PBSE_SISCOMM : sister could not communicate',
                           '60' : ' PBSE_SVRDOWN : req rejected server shutting down',
                           '61' : ' PBSE_CKPSHORT : not all tasks could checkpoint',
                           '62' : ' PBSE_UNKNODE : Named node is not in the list',
                           '63' : ' PBSE_UNKNODEATR : nodeattribute not recognized',
                           '64' : ' PBSE_NONODES : Server has no node list',
                           '65' : ' PBSE_NODENBIG : Node name is too big',
                           '66' : ' PBSE_NODEEXIST : Node name already exists',
                           '67' : ' PBSE_BADNDATVAL : Bad nodeattribute value',
                           '68' : ' PBSE_MUTUALEX : State values are mutually exclusive',
                           '69' : ' PBSE_GMODERR : Error(s) during global modification of nodes',
                           '70' : ' PBSE_NORELYMOM : could not contact Mom',
                           '71' : ' PBSE_NOTSNODE : No timeshare node available',
                           '75' : ' PBSE_RESV_NO_WALLTIME : job reserv lacking walltime',
                           '76' : ' PBSE_JOBNOTRESV : not a reservation job',
                           '77' : ' PBSE_TOOLATE : too late for job reservation',
                           '78' : ' PBSE_IRESVE : internal reservation-system error',
                           '79' : ' unused',
                           '80' : ' PBSE_RESVEXIST : reservation already exists',
                           '81' : ' PBSE_resvFail : reservation failed',
                           '82' : ' PBSE_genBatchReq : batch request generation failed',
                           '83' : ' PBSE_mgrBatchReq : qmgr batch request failed',
                           '84' : ' PBSE_UNKRESVID : unknown reservation ID',
                           '85' : ' PBSE_delProgress : delete already in progress',
                           '86' : ' PBSE_BADTSPEC : bad time specification(s)',
                           '87' : ' PBSE_RESVMSG : so reply_text can send back a msg',
                           '88' : ' PBSE_NOTRESV : not a reservation',
                           '89' : ' PBSE_BADNODESPEC : node(s) specification error',
                           '90' : ' PBSE_UNUSED1 : Licensed CPUs exceeded',
                           '91' : ' PBSE_LICENSEINV : License is invalid',
                           '92' : ' PBSE_RESVAUTH_H : Host machine not authorized to submit reservations',
                           '93' : ' PBSE_RESVAUTH_G : Requestors group not authorized to submit reservations',
                           '94' : ' PBSE_RESVAUTH_U : Requestor not authorized to make reservations',
                           '95' : ' PBSE_R_UID : Bad effective UID for reservation',
                           '96' : ' PBSE_R_GID : Bad effective GID for reservation',
                           '97' : ' PBSE_IBMSPSWITCH : IBM SP Switch error',
                           '98' : ' PBSE_UNUSED2 : Floating License unavailable',
                           '99' : ' PBSE_NOSCHEDULER : Unable to contact Scheduler',
                           '100' : ' PBSE_RESCNOTSTR : resource is not of type string',
                           '107' : ' PBSE_MaxArraySize : max array size exceeded',
                           '108' : ' PBSE_INVALSELECTRESC : resc invalid in select spec',
                           '109' : ' PBSE_INVALJOBRESC : invalid job resource',
                           '110' : ' PBSE_INVALNODEPLACE : node invalid w/ place|select',
                           '111' : ' PBSE_PLACENOSELECT : cannot have place w/o select',
                           '112' : ' PBSE_INDIRECTHOP : too many indirect resc levels',
                           '113' : ' PBSE_INDIRECTBT : target resc undefined',
                           '114' : ' not used',
                           '115' : ' PBSE_NODESTALE : Cannot change state of stale nd',
                           '116' : ' PBSE_DUPRESC : cannot dup resc within a chunk',
                           '117' : ' PBSE_CONNFULL : server connection table full',
                           '118' : ' PBSE_LICENSE_MIN_BADVAL : bad value for pbs_license_min',
                           '119' : ' PBSE_LICENSE_MAX_BADVAL : bad value for pbs_license_max',
                           '120' : ' PBSE_LICENSE_LINGER_BADVAL : bad value for pbs_license_linger_time',
                           '121' : ' PBSE_UNUSED3 : License server is down',
                           '122' : ' PBSE_UNUSED4 : Not allowed action with FLEX licensing',
                           '123' : ' PBSE_BAD_FORMULA : invalid sort formula',
                           '124' : ' PBSE_BAD_FORMULA_KW : invalid keyword in formula',
                           '125' : ' PBSE_BAD_FORMULA_TYPE : invalid resource type in formula',
                           '126' : ' PBSE_BAD_RRULE_YEARLY : reservation duration exceeds 1 year',
                           '127' : ' PBSE_BAD_RRULE_MONTHLY : reservation duration exceeds 1 month',
                           '128' : ' PBSE_BAD_RRULE_WEEKLY : reservation duration exceeds 1 week',
                           '129' : ' PBSE_BAD_RRULE_DAILY : reservation duration exceeds 1 day',
                           '130' : ' PBSE_BAD_RRULE_HOURLY : reservation duration exceeds 1 hour',
                           '131' : ' PBSE_BAD_RRULE_MINUTELY : reservation duration exceeds 1 minute',
                           '132' : ' PBSE_BAD_RRULE_SECONDLY : reservation duration exceeds 1 second',
                           '133' : ' PBSE_BAD_RRULE_SYNTAX : invalid recurrence rule syntax',
                           '134' : ' PBSE_BAD_RRULE_SYNTAX2 : invalid recurrence rule syntax. COUNT/UNTIL required',
                           '135' : ' PBSE_BAD_ICAL_TZ : Undefined timezone info directory',
                           '136' : ' PBSE_HOOKERROR : error encountered related to hooks',
                           '137' : ' PBSE_NEEDQUET : need queue type set',
                           '138' : ' PBSE_ETEERROR : not allowed to alter attribute when eligible_time_enable is off',
                           '139' : ' PBSE_HISTJOBID : History job ID',
                           '140' : ' PBSE_JOBHISTNOTSET : job_history_enable not SET',
                           '141' : ' PBSE_MIXENTLIMS : mixing old and new limit enformcement',
                           '142' : ' PBSE_ENTLIMCT : entity count limit exceeded',
                           '143' : ' PBSE_ENTLIMRESC : entity resource limit exceeded',
                           '144' : ' PBSE_ATVALERANGE : attribute value out of range',
                           '145' : ' PBSE_PROV_HEADERROR : not allowed to set provisioningattributes on head node',
                           '146' : ' PBSE_NODEPROV_NOACTION : cannot modify attribute while node is provisioning',
                           '147' : ' PBSE_NODEPROV : Cannot change state of provisioning node',
                           '148' : ' PBSE_NODEPROV_NODEL : Cannot del node if provisioning',
                           '149' : ' PBSE_NODE_BAD_CURRENT_AOE : current aoe is not one of resources_available.aoe',
                           '153' : ' PBSE_NOLOOPBACKIF : Local host does not have loopback interface configured.',
                           '155' : ' PBSE_IVAL_AOECHUNK : aoe not following chunk rules',
                           '156' : ' PBSE_JOBINRESV_CONFLICT : job and reservation conflict',
                           '157' : ' PBSE_NORUNALTEREDJOB : cannot run altered/moved job',
                           '158' : ' PBSE_HISTJOBDELETED : Job was in F or M state . Its history deleted upon request.',
                           '159' : ' PBSE_NOHISTARRAYSUBJOB : Request invalid for finished array subjob',
                           '160' : ' PBSE_FORCE_QSUB_UPDATE : a qsub action needs to be redone',
                           '161' : ' PBSE_SAVE_ERR : failed to save job or resv to database',
                           '162' : ' PBSE_MAX_NO_MINWT : no max walltime w/o min walltime',
                           '163' : ' PBSE_MIN_GT_MAXWT : min_walltime can not be > max_walltime',
                           '164' : ' PBSE_NOSTF_RESV : There can not be a shrinktofit reservation',
                           '165' : ' PBSE_NOSTF_JOBARRAY : There can not be a shrinkto-fit job array',
                           '166' : ' PBSE_NOLIMIT_RESOURCE : Resource limits can not be set for the resource',
                           '167' : ' PBSE_MOM_INCOMPLETE_HOOK : mom hook not fully transferred',
                           '168' : ' PBSE_MOM_REJECT_ROOT_SCRIPTS : no hook, root job scripts',
                           '169' : ' PBSE_HOOK_REJECT : mom receives a hook rejection',
                           '170' : ' PBSE_HOOK_REJECT_RERUNJOB : hook rejection requiring a job rerun',
                           '171' : ' PBSE_HOOK_REJECT_DELETEJOB : hook rejection requiring a job delete',
                           '172' : ' PBSE_IVAL_OBJ_NAME : invalid object name',
                           '173' : ' PBSE_JOBNBIG : Job name is too long',
                           '174' : ' PBSE_RESCBUSY : Resource is set on an object',
                           '175' : ' PBSE_JOBSCRIPTMAXSIZE : job script max size exceeded',
                           '176' : ' PBSE_BADJOBSCRIPTMAXSIZE : user set size more than 2GB',
                           '177' : ' PBSE_WRONG_RESUME : user tried to resume job with wrong resume signal',
                           '178' : ' PBSE_RESV_NOT_EMPTY : cannot change start time of a nonempty reservation',
                           '179' : ' PBSE_STDG_RESV_OCCR_CONFLICT : cannot change start time of a nonempty reservation',
                           '180' : ' PBSE_SOFTWT_STF : soft_walltime is incompatible with STF jobs',
                           '181' : ' PBSE_RESV_FROM_RESVJOB : Job already in a reservation used to create a reservation',
                           '182' : ' PBSE_RESV_FROM_ARRJOB : Array job used to create a reservation',
                           '183' : ' PBSE_SELECT_NOT_SUBSET : ralter select spec is not a smaller subset of the original',
                           '201' : ' PBSE_RMUNKNOWN : resource unknown',
                           '202' : ' PBSE_RM_ERR_BADPARAM : parameter could not be used',
                           '203' : ' PBSE_RMNOPARAM : a parameter needed did not exist',
                           '204' : ' PBSE_RMNOPARAM : something specified did not exist',
                           '205' : ' PBSE_RMSYSTEM : a system error occured',
                           '206' : ' PBSE_RMPART : only part of reservation made',
                           '208' : ' PBSE_TRYAGAIN : Try the request again later',
                           '209' : ' PBSE_ALPSRELERR : ALPS failed to release the resv',
                           '210' : ' PBSE_JOB_MOVED : Job moved to another server',
                           '211' : ' PBSE_SCHEDEXIST : Scheduler already exists',
                           '212' : ' PBSE_SCHEDEXIST : Scheduler name too long',
                           '213' : ' PBSE_UNKSCHED : sched not in the list',
                           '214' : ' PBSE_SCHED_NO_DEL : can not delete scheduler',
                           '215' : ' PBSE_SCHED_PRIV_EXIST : Scheduler sched_priv directory already exists',
                           '216' : ' PBSE_SCHED_LOG_EXIST : Scheduler sched_log directory already exists',
                           '217' : ' PBSE_ROUTE_QUE_NO_PARTITION : Partition can not be assigned to route queue',
                           '218' : ' PBSE_CANNOT_SET_ROUTE_QUE : Can not set queue type to route',
                           '219' : ' PBSE_QUE_NOT_IN_PARTITION : Queue does not belong to the partition',
                           '220' : ' PBSE_PARTITION_NOT_IN_QUE : Partition does not belong to the queue',
                           '221' : ' PBSE_INVALID_PARTITION_QUE : Invalid partition to the queue',
                           '222' : ' PBSE_ALPS_SWITCH_ERR : ALPS failed to do the suspend/resume',
                           '223' : ' PBSE_SCHED_OP_NOT_PERMITTED : Operation not permitted on default scheduler',
                           '224' : ' PBSE_SCHED_PARTITION_ALREADY_EXISTS : Partition already exists',
                           '225' : ' PBSE_INVALID_MAX_JOB_SEQUENCE_ID : Invalid max_job_sequence_id < 9999999, or > 999999999999',
                           '226' : ' PBSE_SVR_SCHED_JSF_INCOMPAT : Servers job_sort_formula is incompatible with schedulers',
                           '227' : ' PBSE_NODE_BUSY : Node is busy',
                           '228' : ' PBSE_NODE_BUSY : Default partition name is not allowed',
                           '229' : ' PBSE_NODE_BUSY : Finished job did not satisfy dependency',
                           '230' : ' PBSE_SCHEDCONNECTED : unknown error with scheduler connection',
                           '231' : ' PBSE_NOTARRAY_ATTR : Not an array job' }
